@@ -163,7 +163,7 @@ class JWSCore(object):
             sigin = b'.'.join([self.protected.encode('utf-8'), payload])
             self.engine.verify(self.key, sigin, signature)
         except Exception as e:  # pylint: disable=broad-except
-            raise InvalidJWSSignature('Verification failed', repr(e))
+            raise InvalidJWSSignature('Verification failed') from e
         return True
 
 
@@ -180,8 +180,7 @@ class JWS(object):
         :param header_registry: Optional additions to the header registry
         """
         self.objects = dict()
-        if payload:
-            self.objects['payload'] = payload
+        self.objects['payload'] = payload
         self.verifylog = None
         self._allowed_algs = None
         self.header_registry = JWSEHeaderRegistry(JWSHeaderRegistry)
@@ -400,7 +399,8 @@ class JWS(object):
             except ValueError:
                 c = raw_jws.split('.')
                 if len(c) != 3:
-                    raise InvalidJWSObject('Unrecognized representation')
+                    raise InvalidJWSObject('Unrecognized'
+                                           ' representation') from None
                 p = base64url_decode(str(c[0]))
                 if len(p) > 0:
                     o['protected'] = p.decode('utf-8')
@@ -411,7 +411,7 @@ class JWS(object):
             self.objects = o
 
         except Exception as e:  # pylint: disable=broad-except
-            raise InvalidJWSObject('Invalid format', repr(e))
+            raise InvalidJWSObject('Invalid format') from e
 
         if key:
             self.verify(key, alg)
@@ -427,8 +427,7 @@ class JWS(object):
         :param protected: The Protected Header (optional)
         :param header: The Unprotected Header (optional)
 
-        :raises InvalidJWSObject: if no payload has been set on the object,
-                                  or invalid headers are provided.
+        :raises InvalidJWSObject: if invalid headers are provided.
         :raises ValueError: if the key is not a :class:`JWK` object.
         :raises ValueError: if the algorithm is missing or is not provided
          by one of the headers.
@@ -436,9 +435,6 @@ class JWS(object):
          unknown or otherwise not yet implemented.
         """
 
-        if not self.objects.get('payload', None):
-            raise InvalidJWSObject('Missing Payload')
-
         b64 = True
 
         p = dict()
@@ -481,7 +477,8 @@ class JWS(object):
             raise ValueError('"alg" not specified')
 
         c = JWSCore(
-            alg, key, protected, self.objects['payload'], self.allowed_algs
+            alg, key, protected, self.objects.get('payload'),
+            self.allowed_algs
         )
         sig = c.sign()
 
@@ -539,7 +536,7 @@ class JWS(object):
             else:
                 raise InvalidJWSOperation("Can't use compact encoding "
                                           "without protected header")
-            if self.objects.get('payload', False):
+            if self.objects.get('payload'):
                 if self.objects.get('b64', True):
                     payload = base64url_encode(self.objects['payload'])
                 else:
@@ -558,11 +555,11 @@ class JWS(object):
         else:
             obj = self.objects
             sig = dict()
-            if self.objects.get('payload', False):
-                if self.objects.get('b64', True):
-                    sig['payload'] = base64url_encode(self.objects['payload'])
-                else:
-                    sig['payload'] = self.objects['payload']
+            payload = self.objects.get('payload', '')
+            if self.objects.get('b64', True):
+                sig['payload'] = base64url_encode(payload)
+            else:
+                sig['payload'] = payload
             if 'signature' in obj:
                 if not obj.get('valid', False):
                     raise InvalidJWSSignature("No valid signature found")
@@ -590,11 +587,9 @@ class JWS(object):
 
     @property
     def payload(self):
-        if 'payload' not in self.objects:
-            raise InvalidJWSOperation("Payload not available")
         if not self.is_valid:
             raise InvalidJWSOperation("Payload not verified")
-        return self.objects['payload']
+        return self.objects.get('payload')
 
     def detach_payload(self):
         self.objects.pop('payload', None)
